// ------------------------------------------------------------------------------------------------------
// Nunchuck - code the read the intpus from the Nintendo Wii NunChuck
// ------------------------------------------------------------------------------------------------------
//
// The Ninteno Wii Nunchuck is a combination of 2 axis-joystick, 2 pushbuttons, and 3 axis acclerometer
// all sensed and interfaced via the I2C interface. The acclerometer is an ST Microelectronics LIS3L02A 
// MEMS that is capable of 10 bit resolution ( 2g threshold ), while the joystick has 8 bit resolution.
//
// ------------------------------------------------------------------------------------------------------

#include "Arduino.h"        // Arduino common function and defintions
#include "Nunchuck.h"       // my own header file
#include "Wire.h"           // I2C Arduino library

#define NCDevID  0x52       // I2C 7 bit bus address [ 1010010 ]


// data collected / converted from the nunchuck

static uint8_t      readBuffer [ NCDataSize ];   // raw I2C data buffer
static NunchuckData curSample;                   // last sampled / converted read

// ------------------------------------------------------------------------------------------------------
// NunchuckBegin - intitialize the conection with the NunChuck
// ------------------------------------------------------------------------------------------------------

uint8_t NunchuckBegin()
{
  uint8_t wireErr = 0;
  int byteCnt;
    
    
  // start the I2C library
  
  Wire.begin ();
  
            
  // send crypto key enable command ( to allow 3rd party controllers to work )

  Wire.beginTransmission ( NCDevID );
  Wire.write ( (uint8_t) 0xF0 );
  Wire.write ( (uint8_t) 0xAA );
  wireErr = Wire.endTransmission();
  if ( wireErr ) return ( wireErr );


  // write crypto key ( two 6 block writes, one 4 block write )
  
  delay ( 1 );
  Wire.beginTransmission( NCDevID );
  Wire.write ( (uint8_t) 0x40 );
  for ( byteCnt = 0; byteCnt < 6; byteCnt++ ) Wire.write ( (uint8_t) 0x00 );
  wireErr = Wire.endTransmission();
  if ( wireErr ) return ( wireErr );
    
  delay ( 1 );
  Wire.beginTransmission( NCDevID );
  Wire.write ( (uint8_t) 0x40 );
  for ( byteCnt = 6; byteCnt < 12; byteCnt++ ) Wire.write ( (uint8_t) 0x00 );
  wireErr = Wire.endTransmission();
  if ( wireErr ) return ( wireErr ); 
    
  delay ( 1 );
  Wire.beginTransmission( NCDevID );
  Wire.write ( (uint8_t) 0x40 );
  for ( byteCnt = 12; byteCnt < 16; byteCnt++ ) Wire.write ( (uint8_t) 0x00 );
  wireErr = Wire.endTransmission();
  if ( wireErr ) return ( wireErr );   
   
   
  // send a zero to the device to resample and store in register for next poll
   
  Wire.beginTransmission ( NCDevID );
  Wire.write ( ( uint8_t ) 0x00 );
  wireErr = Wire.endTransmission ();
  if ( wireErr ) return ( wireErr );
  delay ( 1 );
   
   return ( 0 );
}

// ------------------------------------------------------------------------------------------------------
// NunchuckUpdate - obtain the current sensor values of the Nunchuck
// ------------------------------------------------------------------------------------------------------

uint8_t NunchuckUpdate()
{
  uint8_t wireErr = 0;
  
  
  // request data from device
  
  uint8_t numBytesRead = Wire.requestFrom ( NCDevID, NCDataSize );
  if ( numBytesRead != NCDataSize ) return ( 255 );

  
  
  // read the data and wait until all bytes have been read in
  // note that raw data must be XOR'ed by 0x17 + 0x17
  
  for ( int bufCnt = 0; bufCnt < NCDataSize; bufCnt++ )
  {
     uint8_t rawByte       = Wire.read();
     readBuffer [ bufCnt ] = ( rawByte ^ 0x17) + 0x17;
   }
  
  #if 0 
   Serial.print ( readBuffer [0] , DEC ); Serial.print ( " " );
   Serial.print ( readBuffer [1] , DEC ); Serial.print ( " " );
   Serial.print ( readBuffer [2] , DEC ); Serial.print ( " " );
   Serial.print ( readBuffer [3] , DEC ); Serial.print ( " " );
   Serial.print ( readBuffer [4] , DEC ); Serial.print ( " " );
   Serial.print ( readBuffer [5] , DEC ); Serial.println ( " " );
   #endif
   
   // convert the raw data in usable values
   
   curSample.xStick  =   readBuffer [ NCStickX ];
   curSample.yStick  =   readBuffer [ NCStickY ];
   curSample.xAccel  = ( readBuffer [ NCAccelX ] << 2 ) | ( ( readBuffer [ NCButtons ] >> 2 ) & 0x03 );
   curSample.yAccel  = ( readBuffer [ NCAccelY ] << 2 ) | ( ( readBuffer [ NCButtons ] >> 4 ) & 0x03 );
   curSample.zAccel  = ( readBuffer [ NCAccelZ ] << 2 ) | ( ( readBuffer [ NCButtons ] >> 6 ) & 0x03 );
   curSample.zButton = ( ( readBuffer [ NCButtons ] & 0x01 ) ? false : true );
   curSample.cButton = ( ( readBuffer [ NCButtons ] & 0x02 ) ? false : true );
   
   uint8_t butState = readBuffer [ NCButtons ] & 0x03;
   

   
   // send a zero to the device to resample and store in register for next poll
   
   Wire.beginTransmission ( NCDevID );
   Wire.write ( ( uint8_t ) 0x00 );
   wireErr = Wire.endTransmission ();
   if ( wireErr ) return ( wireErr );
   
   return ( 0 );  // no error
}


// ------------------------------------------------------------------------------------------------------
// NunchuckGet -- get the last sampled nunchuck value
// ------------------------------------------------------------------------------------------------------

NunchuckData * NunchuckGet ()
{
  return ( & curSample );
}


